package com.xsnb.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.xsnb.common.EventType;
import com.xsnb.common.constants.JwtConstants;
import com.xsnb.common.constants.TokenConstants;
import com.xsnb.common.domain.User;
import com.xsnb.common.domain.XmlData;
import com.xsnb.common.domain.response.RespJwt;
import com.xsnb.common.exception.WechatQrCodeException;
import com.xsnb.common.utils.JwtUtils;
import com.xsnb.system.domain.WxUserLog;
import com.xsnb.system.mapper.UserMapper;
import com.xsnb.system.mapper.WxUserLogMapper;
import com.xsnb.system.service.WechatService;
import com.xsnb.system.utils.GzhUtiles;
import com.xsnb.system.utils.HttpUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Service
@Log4j2
public class WechatServiceImpl implements WechatService {


    @Autowired
    private RedisTemplate<String, String> redisTemplate;
    @Autowired
    private HttpUtil httpUtil;
    @Autowired
    private GzhUtiles gzhUtiles;

    @Autowired
    private UserMapper userMapper;
    @Autowired
    private WxUserLogMapper mapper;


    /**
     * 获取微信二维码
     *
     * @return /**
     * 获取微信二维码信息
     * @return 包含二维码 ticket 和 codeUrl 的 Map
     */
    @Override
    public Map<String, String> getQRCode() {
        try {
            // 获取公众号 AccessToken
            String gzhAccessToken = gzhUtiles.getToken();
            // 构建请求URL
            String createQRCodeUrl = "https://api.weixin.qq.com/cgi-bin/qrcode/create?access_token=" + gzhAccessToken;
            // 构建二维码数据
            String jsonData = "{\"expire_seconds\": 600, \"action_name\": \"QR_STR_SCENE\", \"action_info\": {\"scene\": {\"scene_str\": \"login\"}}}";
            // 发起POST请求获取二维码 ticket
            Map<String, Object> ticketMap = httpUtil.doPost(createQRCodeUrl, jsonData, 600);
            String ticket = String.valueOf(ticketMap.get("ticket"));
            if (ticket != null) {
                // 构建二维码显示URL
                String codeUrl = "https://mp.weixin.qq.com/cgi-bin/showqrcode?ticket=" + ticket;
                HashMap<String, String> wxMap = new HashMap<>();
                wxMap.put("uuid", ticket);
                wxMap.put("codeUrl", codeUrl);
                return wxMap;
            }
        } catch (Exception e) {
            // 发生异常时抛出自定义异常
            throw new WechatQrCodeException();
        }

        // 默认返回空 Map
        return Collections.emptyMap();
    }

    @Override
    public void validationWeCat(String requestBody) {
        XmlData xmlData = GzhUtiles.jxXml(requestBody);
        WxUserLog wxUserLog = new WxUserLog().builder()
                .createTime(String.valueOf(xmlData.getCreateTime()))
                .event(xmlData.getEvent())
                .eventKey(xmlData.getEventKey())
                .fromUserName(xmlData.getFromUserName())
                .msgType(xmlData.getMsgType())
                .ticket(xmlData.getTicket())
                .toUserName(xmlData.getToUserName())
                .build();
        if (xmlData!=null){

            int insert = mapper.insert(wxUserLog);
            if (insert>0){
                System.out.println("添加成功");
            }
        }

        log.info("获取的xmlData:{}", JSONObject.toJSONString(xmlData));
        Objects.requireNonNull(xmlData, "xml格式校验失败-null");
        Objects.requireNonNull(xmlData.getEvent(), "xml格式校验失败-eventTypeNull");
        Objects.requireNonNull(xmlData.getFromUserName(), "xml格式校验失败-eventTypeNull");
        String eventType = xmlData.getEvent();
        String openId = xmlData.getFromUserName();
        String eventKey = xmlData.getEventKey();
        //todo.1 在公众号点击关注不在进行用户的注册登录，仅仅在用户进行官网扫码时间进行用户的相关操作
        //登录注册
        //用户已关注扫描临时二维码或未关注扫描临时二维码进行关注
        if ((EventType.Login.getValue().equals(eventType) && "login".equals(eventKey)) || (EventType.SUBSCRIBE.getValue().equals(eventType) && "qrscene_login".equals(eventKey))) {
            String ticket = xmlData.getTicket();
            login(ticket, openId);
        }

    }


    public RespJwt login(String ticket, String openId) {
        log.info("拿到的uuid是:{} ,wxOpenID是:{}........",ticket,openId);
        redisTemplate.opsForValue().set(ticket + "state", String.valueOf(1));
        //用openid查询用户信息
        QueryWrapper<User> wrapper = new QueryWrapper<>();
        QueryWrapper<User> id = wrapper.eq("open_id", openId);
        User user = userMapper.selectOne(id);
        if (user == null) {
            //用户不存在，进行注册
            log.info("用户不存在，进行注册");
            //todo.2 暂时不进行注册，直接进行登录
        }
        user.setWx_uid(ticket);
        int update = userMapper.update(user, id);
        if (update == 0) {
            log.info("用户不存在，进行注册");
        }
        HashMap<String, Object> map = new HashMap<>();
        String userkey = UUID.randomUUID().toString().replaceAll("-", "");
        map.put(JwtConstants.USER_KEY, userkey);
        map.put(JwtConstants.DETAILS_USER_ID, user.getId());
        // 生成token
        String token = JwtUtils.createToken(map);
        //存token
        redisTemplate.opsForValue().set(
                TokenConstants.LOGIN_TOKEN_KEY + userkey,
                JSONObject.toJSONString(user),
                TokenConstants.EXPIRATION,
                TimeUnit.MINUTES
        );
        RespJwt respJwt = new RespJwt();
        respJwt.setToken(token);
        respJwt.setEidTime("720MIN");
        return respJwt;
    }


    /**
     * 轮询查看微信的登录状态，是否可以登录，返回token
     *
     * @param ticket 微信扫码的uuid
     * @return 包含登录状态信息的Map
     */
    public RespJwt checkLogin(String ticket) {
        log.info("checkLogin+二维码的ticket传参:{}", ticket);
        RespJwt respJwt = null;
        Boolean ticketState = redisTemplate.hasKey(ticket + "state");//存在说明已经扫码
        //通过uuid到user里拿openId
        QueryWrapper<User> wrapper = new QueryWrapper<>();
        QueryWrapper<User> id = wrapper.eq("wx_uid", ticket);
        User user = userMapper.selectOne(id);
        if (user == null) {
            //用户不存在，进行注册
            log.info("用户不存在，进行注册");
            //todo.2 暂时不进行注册，直接进行登录
        }

        if (ticketState) {
            String s = redisTemplate.opsForValue().get(ticket + "state");
            if ("1".equals(s)) {
                //进行登录
                if (user != null) {
                    respJwt = login(ticket, user.getOpenId());
                }else {

                    log.info("user为null,用户不存在，进行注册");
                }
            }
        }
        return respJwt;
    }
}
